--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
--
--  file:    mapinfo.lua
--  brief:   map configuration loader
--  author:  Dave Rodgers, Lurker, Smoth
--
--  Copyright (C) 2008.
--  Licensed under the terms of the GNU GPL, v2 or later.
--
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

-- Special table for map config files
--
--   Map.fileName
--   Map.fullName
--   Map.configFile


--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

local function IsTdf(text)
  local s, e, c = text:find('(%S)') -- first non-space character
  -- TDF files should start with:
  --   '[name]' - section header
  --   '/*'     - comment
  --   '//      - comment
  return ((c == '[') or (c == '/'))
end


--------------------------------------------------------------------------------

local configFile = Map.configFile
local mapcfg
local waterheight

if VFS.FileExists("config/mapconfig.lua") then
	mapcfg = VFS.Include("config/mapconfig.lua")
	else
		  error("missing file: config/mapconfig.lua")
end

local WaterDefs

local configSource, err = VFS.LoadFile(configFile)
if (configSource == nil) then
  error(configFile .. ': ' .. err)
end


--------------------------------------------------------------------------------

local mapInfo, err

if (IsTdf(configSource)) then
  local mapTdfParser = VFS.Include('maphelper/parse_tdf_map.lua')
  mapInfo, err = mapTdfParser(configSource)
  if (mapInfo == nil) then
    error(configFile .. ': ' .. err)
  end
else
  local chunk, err = loadstring(configSource, configFile)
  if (chunk == nil) then
    error(configFile .. ': ' .. err)
  end
  mapInfo = chunk()
end

mapInfo.smf ={
		minheight	=	0,
		maxheight	=	0,
}
--------------------------------------------------------------------------------

if (Spring.GetMapOptions and mapInfo.defaultoptions) then
  local optsFunc = VFS.Include('maphelper/applyopts.lua')
  optsFunc(mapInfo)
end

if Spring.GetMapOptions then
	
	-- Get some defaults
	mapInfo.atmosphere.minwind	=  	mapcfg.minwind
	mapInfo.atmosphere.maxwind	=  	mapcfg.maxwind
	mapInfo.tidalstrength		=  	mapcfg.tidalstrength
	mapInfo.gravity				=  	mapcfg.gravity
	mapInfo.maxmetal			=  	mapcfg.maxmetal
	mapInfo.extractorradius		=  	mapcfg.extractorradius
	mapInfo.maphardness			=  	mapcfg.maphardness
	mapInfo.autoshowmetal		=   mapcfg.autoshowmetal
	
	if (Spring.GetMapOptions().inv == "1" ) then	
		mapInfo.smf.minheight 		= 	mapcfg.maxheight
		mapInfo.smf.maxheight 		=  	mapcfg.minheight
		waterheight					=	mapcfg.invertedwaterheight
		else
			mapInfo.smf.minheight 		= 	mapcfg.minheight
			mapInfo.smf.maxheight 		=  	mapcfg.maxheight
			waterheight					=	mapcfg.waterheight
	end

	if (Spring.GetMapOptions().metal == "0" ) then
		mapInfo.maxmetal = 0.3
	elseif (Spring.GetMapOptions().metal == "2" ) then
		mapInfo.maxmetal = 0.84
	else
		mapInfo.maxmetal = mapcfg.maxmetal
	end
	
	if (Spring.GetMapOptions().wind == "0" ) then
		mapInfo.atmosphere.maxwind = 14
		mapInfo.atmosphere.minwind = 2
	elseif (Spring.GetMapOptions().wind == "2" ) then
		mapInfo.atmosphere.maxwind = 40
		mapInfo.atmosphere.minwind = 8
	else
		mapInfo.atmosphere.maxwind = mapcfg.maxwind
		mapInfo.atmosphere.minwind = mapcfg.minwind
	end
	
		if (Spring.GetMapOptions().wc == "1")	then
			if (Spring.GetMapOptions().alt == "1") then
				AtmoSDefs = mapcfg.night.weather.snow
				WaterDefs = mapcfg.night.water
				else 
					AtmoSDefs = mapcfg.day.weather.snow
					WaterDefs = mapcfg.day.water		
			end
			else	-- clear skies
				if (Spring.GetMapOptions().alt == "1") then
					AtmoSDefs = mapcfg.night.weather.clear
					WaterDefs = mapcfg.night.water
					else 
					--Settings day.
						AtmoSDefs = mapcfg.day.weather.clear
						WaterDefs = mapcfg.day.water
				end
		end
		
		if(Spring.GetMapOptions().wc) then
		Spring.Echo(Spring.GetMapOptions().wc .. "  OLIOLIO  ")
		end
		
		
		if VFS.FileExists("maps/" .. AtmoSDefs.skybox) then
		mapInfo.atmosphere.skybox		=	AtmoSDefs.skybox
			else
			mapInfo.atmosphere.skybox		=	''
		end
		
		if VFS.FileExists("maps/" .. AtmoSDefs.detailTex) then
		mapInfo.resources.detailTex 		= 	AtmoSDefs.detailTex
		end
		
		mapInfo.atmosphere.cloudcolor			=	AtmoSDefs.cloudcolor
		mapInfo.atmosphere.suncolor				=	AtmoSDefs.suncolor
		mapInfo.atmosphere.clouddensity			=	AtmoSDefs.clouddensity
		mapInfo.atmosphere.skycolor				=	AtmoSDefs.skycolor
		mapInfo.atmosphere.fogcolor				=	AtmoSDefs.fogcolor
		mapInfo.atmosphere.fogstart				=	AtmoSDefs.fogstart

		mapInfo.lighting.sundir					=	AtmoSDefs.sundir	
		mapInfo.lighting.groundambientcolor		=	AtmoSDefs.groundambientcolor	
		mapInfo.lighting.grounddiffusecolor		=	AtmoSDefs.grounddiffusecolor
		mapInfo.lighting.groundshadowdensity	=	AtmoSDefs.groundshadowdensity

		mapInfo.lighting.unitambientcolor		=	AtmoSDefs.unitambientcolor
		mapInfo.lighting.unitdiffusecolor		=	AtmoSDefs.unitdiffusecolor
		mapInfo.lighting.unitshadowdensity		=	AtmoSDefs.unitshadowdensity
		mapInfo.lighting.specularsuncolor		=	AtmoSDefs.specularsuncolor

		
	
	if (Spring.GetMapOptions().wet == "1" ) then
		mapInfo.smf.minheight 	= 	mapInfo.smf.minheight - waterheight
		mapInfo.smf.maxheight	=  	mapInfo.smf.maxheight - waterheight
		mapInfo.voidwater		=	false		
		
		mapInfo.water.shorewaves 			= WaterDefs.shorewaves
		mapInfo.water.forcerendering 		= WaterDefs.forcerendering
		mapInfo.water.absorb				= WaterDefs.absorb
		mapInfo.water.ambientfactor 		= WaterDefs.ambientfactor
		mapInfo.water.basecolor				= WaterDefs.basecolor
		mapInfo.water.blurbase 				= WaterDefs.blurbase
		mapInfo.water.blurexponent 			= WaterDefs.blurexponent
		mapInfo.water.diffusecolor 			= WaterDefs.diffusecolor
		mapInfo.water.diffusefactor 		= WaterDefs.diffusefactor
		mapInfo.water.fresnelmin 			= WaterDefs.fresnelmin
		mapInfo.water.fresnelmax 			= WaterDefs.fresnelmax
		mapInfo.water.fresnelpower 			= WaterDefs.fresnelpower
		mapInfo.water.mincolor				= WaterDefs.mincolor
		mapInfo.water.perlinstartfreq 		= WaterDefs.perlinstartfreq
		mapInfo.water.perlinlacunarity 		= WaterDefs.perlinlacunarity
		mapInfo.water.perlinamplitude 		= WaterDefs.perlinamplitude		
		mapInfo.water.planecolor			= WaterDefs.planecolor
		mapInfo.water.reflectiondistortion 	= WaterDefs.reflectiondistortion
		mapInfo.water.specularcolor 		= WaterDefs.specularcolor
		mapInfo.water.specularfactor 		= WaterDefs.specularfactor
		mapInfo.water.specularpower 		= WaterDefs.specularpower
		mapInfo.water.surfacecolor			= WaterDefs.surfacecolor
		mapInfo.water.surfacealpha			= WaterDefs.surfacealpha
		else
			mapInfo.voidwater		=	true
	end
end
--------------------------------------------------------------------------------
--------------------------------------------------------------------------------

return mapInfo

--------------------------------------------------------------------------------
--------------------------------------------------------------------------------
